﻿namespace TabsStudioMarker
{
    class Commands
    {
        public Commands(TabsStudioExt.ITabsStudioEngine engine_, EnvDTE80.DTE2 dte_, CommandsImpl commandsImpl_)
        {
            engine = engine_;
            dte = dte_;
            commandsImpl = commandsImpl_;
            if (!new TabsStudioExt.Util.VsVersion(dte).IsExtension())
                RegisterCommands();
        }

        public void QueryStatus(string CmdName, EnvDTE.vsCommandStatusTextWanted NeededText,
                ref EnvDTE.vsCommandStatus StatusOption, ref System.Object CommandText)
        {
            if (NeededText != EnvDTE.vsCommandStatusTextWanted.vsCommandStatusTextWantedNone || CmdName == null)
                return;

            if (CmdName == "TabsStudio.Connect.MarkerRemoveAllHighlighting" ||
                CmdName == "TabsStudio.Connect.MarkerCloseAllButHighlighted" ||
                CmdName == "TabsStudio.Connect.MarkerToggleTabHighlighting" ||
                CmdName == "TabsStudio.Connect.MarkerNextTabHighlightingColor")
            {
                StatusOption = (EnvDTE.vsCommandStatus)SupportedAndEnabled();
            }
        }

        public void Exec(string CmdName, EnvDTE.vsCommandExecOption ExecuteOption, ref System.Object VariantIn,
            ref System.Object VariantOut, ref bool Handled)
        {
            Handled = false;
            if (ExecuteOption != EnvDTE.vsCommandExecOption.vsCommandExecOptionDoDefault || CmdName == null)
                return;

            if (CmdName == "TabsStudio.Connect.MarkerRemoveAllHighlighting")
            {
                commandsImpl.RemoveAllHighlighting();
                Handled = true;
            }
            if (CmdName == "TabsStudio.Connect.MarkerCloseAllButHighlighted")
            {
                commandsImpl.CloseAllButHighlighted();
                Handled = true;
            }
            if (CmdName == "TabsStudio.Connect.MarkerToggleTabHighlighting")
            {
                commandsImpl.ToggleTabHighlighting();
                Handled = true;
            }
            if (CmdName == "TabsStudio.Connect.MarkerNextTabHighlightingColor")
            {
                commandsImpl.NextTabHighlightingColor();
                Handled = true;
            }
        }

        private void RegisterCommands()
        {
            EnvDTE80.Commands2 commands = dte.Commands as EnvDTE80.Commands2;
            if (commands == null)
                return;

            RegisterCommand(commands, engine.GetAddIn(), "MarkerRemoveAllHighlighting", "Remove All Highlighting",
                "Remove All Highlighting");
            RegisterCommand(commands, engine.GetAddIn(), "MarkerCloseAllButHighlighted", "Close All But Highlighted",
                "Close All But Highlighted");
            RegisterCommand(commands, engine.GetAddIn(), "MarkerToggleTabHighlighting", "Toggle Tab Highlighting",
                "Toggle Tab Highlighting");
            RegisterCommand(commands, engine.GetAddIn(), "MarkerNextTabHighlightingColor", "Next Tab Highlighting Color",
                "Next Tab Highlighting Color");

            //commands.Item("TabsStudio.Connect.NavigateToNextTab", 0).Delete();
        }

        private void RegisterCommand(EnvDTE80.Commands2 commands, EnvDTE.AddIn addin,
            string name, string buttonText, string tooltip)
        {
            try
            {
                object[] contextGUIDS = new object[] { };
                commands.AddNamedCommand2(addin, name, buttonText, tooltip,
                    true, 36, ref contextGUIDS,
                    (int)SupportedAndEnabled(),
                    (int)EnvDTE80.vsCommandStyle.vsCommandStylePictAndText,
                    EnvDTE80.vsCommandControlType.vsCommandControlTypeButton);
            }
            catch (System.ArgumentException)
            {
                // Command already exists.
            }
        }

        private int SupportedAndEnabled()
        {
            return (int)EnvDTE.vsCommandStatus.vsCommandStatusSupported +
                (int)EnvDTE.vsCommandStatus.vsCommandStatusEnabled;
        }

        private TabsStudioExt.ITabsStudioEngine engine;
        private EnvDTE80.DTE2 dte;
        private CommandsImpl commandsImpl;
    }
}
