﻿namespace TabsStudioMarker
{
    class CommandsImpl
    {
        public CommandsImpl(TabsStudioExt.ITabsStudioEngine engine_, EnvDTE80.DTE2 dte, Model model_)
        {
            engine = engine_;
            model = model_;
            enumWindowFrames = new EnumWindowFrames(dte);
        }

        public void RemoveAllHighlighting()
        {
            System.Windows.MessageBoxResult res = System.Windows.MessageBox.Show(
                "Do you want to remove highlighting from all tabs?", "Marker",
                System.Windows.MessageBoxButton.YesNo,
                System.Windows.MessageBoxImage.Warning);
            if (res == System.Windows.MessageBoxResult.No)
                return;

            model.RemoveAllHighlighting();
        }

        public void CloseAllButHighlighted()
        {
            TabsStudioExt.Tabs tabs = FindSelectedTabs();
            if (tabs != null)
                CloseAllButHighlighted(tabs);
        }

        public void ToggleTabHighlighting()
        {
            TabsStudioExt.Tab tab = FindSelectedTab();
            if (tab != null)
                ToggleTabHighlighting(tab.TabItem);
        }

        public void NextTabHighlightingColor()
        {
            TabsStudioExt.Tab tab = FindSelectedTab();
            if (tab != null)
            {
                uint newIndex = (model.GetHighlightIndex(tab.TabItem) + 1) % 6;
                model.SetHighlightIndex(tab.TabItem, newIndex);
            }
        }

        public void CloseAllButHighlighted(TabsStudioExt.Tabs tabs)
        {
            System.Collections.Generic.List<EnvDTE.Window> windows = new System.Collections.Generic.List<EnvDTE.Window>();
            foreach (TabsStudioExt.Tab tab in tabs.TabList)
            {
                if (model.GetHighlightIndex(tab.TabItem) == 0)
                {
                    foreach (TabsStudioExt.TabExtension ext in tab.TabExtensions)
                        windows.Add(ext.Window);
                }
            }

            System.Collections.Generic.List<WindowFrame> frames = enumWindowFrames.GetWindowFrames();
            foreach (EnvDTE.Window w in windows)
            {
                foreach (WindowFrame frame in frames)
                {
                    if (frame.w == w)
                    {
                        bool userCancelled = frame.frame.CloseFrame(
                            (uint)Microsoft.VisualStudio.Shell.Interop.__FRAMECLOSE.FRAMECLOSE_PromptSave) !=
                                Microsoft.VisualStudio.VSConstants.S_OK;
                        if (userCancelled)
                            return;
                        break;
                    }
                }
            }
        }

        public void ToggleTabHighlighting(System.Windows.Controls.TabItem tabItem)
        {
            model.SetHighlightIndex(tabItem, model.GetHighlightIndex(tabItem) > 0 ? 0u : 1u);
        }

        private TabsStudioExt.Tabs FindSelectedTabs()
        {
            foreach (TabsStudioExt.Tabs tabs in engine.GetTabs())
            {
                if (tabs.IsGroupSelected)
                    return tabs;
            }
            return null;
        }

        private TabsStudioExt.Tab FindSelectedTab()
        {
            TabsStudioExt.Tabs tabs = FindSelectedTabs();
            if (tabs != null)
            {
                foreach (TabsStudioExt.Tab tab in tabs.TabList)
                {
                    if (tab.IsTabSelected)
                        return tab;
                }
            }
            return null;
        }

        private TabsStudioExt.ITabsStudioEngine engine;
        private Model model;
        private EnumWindowFrames enumWindowFrames;
    }
}
