﻿namespace TabsStudioMarker
{
    class Model
    {
        public Model(TabsStudioExt.ITabsStudioEngine engine_, EnvDTE80.DTE2 dte, Options options_)
        {
            engine = engine_;
            options = options_;
            openDocuments = new OpenDocuments();
            engine.TabExtensionCreated += OnTabExtensionCreated;
            engine.TabExtensionDestroyed += OnTabExtensionDestroyed;
            engine.TabExtensionRenamed += OnTabExtensionRenamed;
            highlightedDocuments = new System.Collections.Generic.List<TabHighlightState>();
            solutionPath = new SolutionPath(dte);
        }

        public void Close()
        {
            engine.TabExtensionCreated -= OnTabExtensionCreated;
            engine.TabExtensionDestroyed -= OnTabExtensionDestroyed;
            engine.TabExtensionRenamed -= OnTabExtensionRenamed;
        }

        public void SetHighlightIndex(System.Windows.Controls.TabItem tab, uint colorIndex)
        {
            Properties.SetHighlightIndex(tab, colorIndex);

            System.Collections.Generic.List<WindowDocument> extensions = openDocuments.FindDocuments(tab);
            if (extensions.Count == 0)
                return;

            RemoveHighlightedDocuments(extensions);
            if (colorIndex > 0)
            {
                System.Collections.Generic.List<string> paths = new System.Collections.Generic.List<string>();
                foreach (WindowDocument wd in extensions)
                    paths.Add(solutionPath.RelativePath(wd));
                highlightedDocuments.Add(new TabHighlightState(paths, colorIndex));
            }
        }

        public uint GetHighlightIndex(System.Windows.Controls.TabItem tab)
        {
            return Properties.GetHighlightIndex(tab);
        }

        public void RemoveAllHighlighting()
        {
            foreach (TabsStudioExt.Tabs tabs in engine.GetTabs())
            {
                foreach (TabsStudioExt.Tab tab in tabs.TabList)
                {
                    if (Properties.GetHighlightIndex(tab.TabItem) > 0)
                        Properties.SetHighlightIndex(tab.TabItem, 0);
                }
            }
            highlightedDocuments.Clear();
        }

        public System.Collections.Generic.List<TabHighlightState> GetTabs()
        {
            return highlightedDocuments;
        }

        public void SetTabs(System.Collections.Generic.List<TabHighlightState> newState)
        {
            if (newState == null)
                highlightedDocuments.Clear();
            else
                highlightedDocuments = newState;
        }

        private void OnTabExtensionCreated(object sender, TabsStudioExt.TabExtensionEventArgs e)
        {
            openDocuments.OnTabExtensionCreated(e);

            uint colorIndex = FindHighlighted(new WindowDocument(e.TabExtension.Window));
            if (colorIndex > 0)
                SetHighlightIndex(e.Tab.TabItem, colorIndex);
        }

        private void OnTabExtensionDestroyed(object sender, TabsStudioExt.TabExtensionEventArgs e)
        {
            if (!options.maintainHighlightStatus)
            {
                WindowDocument s = openDocuments.FindDocument(e.TabExtension.Window);
                if (s != null)
                    RemoveHighlightedDocument(s);
            }

            openDocuments.OnTabExtensionDestroyed(e);
        }

        private void OnTabExtensionRenamed(object sender, TabsStudioExt.TabExtensionEventArgs e)
        {
            WindowDocument oldId = openDocuments.FindDocument(e.TabExtension.Window);
            if (oldId != null)
                RenameHighlightedDocument(oldId, solutionPath.RelativePath(new WindowDocument(e.TabExtension.Window)));

            openDocuments.OnTabExtensionRenamed(e);
        }

        private uint FindHighlighted(WindowDocument id)
        {
            foreach (TabHighlightState tab in highlightedDocuments)
            {
                foreach (string s in tab.extensions)
                {
                    if (DocumentsEqual(id, s))
                        return tab.highlightIndex;
                }
            }
            return 0;
        }

        private void RemoveHighlightedDocuments(System.Collections.Generic.List<WindowDocument> extensions)
        {
            foreach (WindowDocument s in extensions)
                RemoveHighlightedDocument(s);
        }

        private void RemoveHighlightedDocument(WindowDocument id)
        {
            foreach (TabHighlightState tab in highlightedDocuments)
            {
                foreach (string s in tab.extensions)
                {
                    if (DocumentsEqual(id, s))
                    {
                        tab.extensions.Remove(s);
                        if (tab.extensions.Count == 0)
                            highlightedDocuments.Remove(tab);
                        return;
                    }
                }
            }
        }

        private void RenameHighlightedDocument(WindowDocument oldId, string newId)
        {
            foreach (TabHighlightState tab in highlightedDocuments)
            {
                for (int i = 0; i < tab.extensions.Count; ++i)
                {
                    if (DocumentsEqual(oldId, tab.extensions[i]))
                    {
                        tab.extensions[i] = newId;
                        return;
                    }
                }
            }
        }

        private bool DocumentsEqual(WindowDocument openDocument, string highlightedDocument)
        {
            return string.Compare(highlightedDocument, openDocument.document, true) == 0 || 
                string.Compare(highlightedDocument, solutionPath.RelativePath(openDocument), true) == 0;
        }

        private TabsStudioExt.ITabsStudioEngine engine;
        private Options options;
        private OpenDocuments openDocuments;
        private System.Collections.Generic.List<TabHighlightState> highlightedDocuments;
        private SolutionPath solutionPath;
    }
}
