﻿namespace TabsStudioMarker
{
    class SolutionEventsSink : Microsoft.VisualStudio.Shell.Interop.IVsSolutionEvents
    {
        public SolutionEventsSink(SolutionStateSaver saver, EnvDTE80.DTE2 dte)
        {
            this.saver = saver;
            this.dte = dte;

            if (IsSolutionOpen())
                OnSolutionOpened();
        }

        public int OnAfterCloseSolution(System.Object pUnkReserved)
        {
            return Microsoft.VisualStudio.VSConstants.S_OK;
        }

        public int OnAfterLoadProject(Microsoft.VisualStudio.Shell.Interop.IVsHierarchy pStubHierarchy,
            Microsoft.VisualStudio.Shell.Interop.IVsHierarchy pRealHierarchy)
        {
            return Microsoft.VisualStudio.VSConstants.S_OK;
        }

        public int OnAfterOpenProject(Microsoft.VisualStudio.Shell.Interop.IVsHierarchy pHierarchy,
            int fAdded)
        {
            return Microsoft.VisualStudio.VSConstants.S_OK;
        }

        public int OnAfterOpenSolution(System.Object pUnkReserved, int fNewSolution)
        {
            OnSolutionOpened();
            return Microsoft.VisualStudio.VSConstants.S_OK;
        }

        private void OnSolutionOpened()
        {
            try
            {
                saver.Load();
            }
            catch (System.Exception ex)
            {
                System.Windows.MessageBox.Show(ex.Message + System.Environment.NewLine + ex.StackTrace,
                    "Marker (Unhandled exception)",
                    System.Windows.MessageBoxButton.OK, System.Windows.MessageBoxImage.Error);
            }
        }

        public int OnBeforeCloseProject(Microsoft.VisualStudio.Shell.Interop.IVsHierarchy pHierarchy,
            int fRemoved)
        {
            return Microsoft.VisualStudio.VSConstants.S_OK;
        }

        public int OnBeforeCloseSolution(System.Object pUnkReserved)
        {
            try
            {
                saver.Save();
            }
            catch (System.Exception ex)
            {
                System.Windows.MessageBox.Show(ex.Message + System.Environment.NewLine + ex.StackTrace,
                    "Marker (Unhandled exception)",
                    System.Windows.MessageBoxButton.OK, System.Windows.MessageBoxImage.Error);
            }
            return Microsoft.VisualStudio.VSConstants.S_OK;
        }

        public int OnBeforeUnloadProject(Microsoft.VisualStudio.Shell.Interop.IVsHierarchy pRealHierarchy,
            Microsoft.VisualStudio.Shell.Interop.IVsHierarchy pStubHierarchy)
        {
            return Microsoft.VisualStudio.VSConstants.S_OK;
        }

        public int OnQueryCloseProject(Microsoft.VisualStudio.Shell.Interop.IVsHierarchy pHierarchy,
            int fRemoving, ref int pfCancel)
        {
            return Microsoft.VisualStudio.VSConstants.S_OK;
        }

        public int OnQueryCloseSolution(System.Object pUnkReserved, ref int pfCancel)
        {
            return Microsoft.VisualStudio.VSConstants.S_OK;
        }

        public int OnQueryUnloadProject(Microsoft.VisualStudio.Shell.Interop.IVsHierarchy pRealHierarchy,
            ref int pfCancel)
        {
            return Microsoft.VisualStudio.VSConstants.S_OK;
        }

        private bool IsSolutionOpen()
        {
            return dte != null && dte.Solution != null && dte.Solution.IsOpen;
        }

        private EnvDTE80.DTE2 dte;
        private SolutionStateSaver saver;
    }
}
